@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022-2024 Univention GmbH
+ * Copyright (c) 2022-2025 Univention GmbH
  * Copyright (c) 2009-2015 Emmanuel Dreyfus
  * All rights reserved.
  *
@@ -32,6 +32,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <string.h>
@@ -44,6 +45,10 @@
 
 #include <jansson.h>
 
+#ifdef HACK
+#include <yder.h>
+#endif
+
 #include <sasl/sasl.h>
 #include <sasl/saslplug.h>
 #include <sasl/saslutil.h>
@@ -381,7 +386,7 @@ static int oauth_server_mech_step(
 	sasl_out_params_t *oparams
 ) {
 	oauth_serv_context_t *ctx = (oauth_serv_context_t *)conn_context;
-	// oauth_glob_context_t *gctx;
+	oauth_glob_context_t *gctx;
 	char *authzid = NULL;
 	const char *authcid = NULL;
 	char *auth_msg = NULL;
@@ -407,7 +412,8 @@ static int oauth_server_mech_step(
 		params->utils->seterror(params->utils->conn, 0, "could not get SASL_SSF_EXTERNAL");
 		return SASL_BADPARAM;
 	}
-	if (!((ssfp ? *ssfp : 0) >= 256)) {
+	gctx = ctx->glob_context;
+	if (gctx->tls_required && !((ssfp ? *ssfp : 0) >= 256)) {
 		params->utils->seterror(params->utils->conn, 0, "TLS required!");
 		return SASL_ENCRYPT;
 	}
@@ -572,7 +578,8 @@ static void oauth_server_mech_free(
 static sasl_server_plug_t oauth_server_plugin = {
 	"OAUTHBEARER",  /* mech_name */
 	0,  /* max_ssf */
-	SASL_SEC_NOANONYMOUS,  /* security_flags */
+	// setting SASL_SEC_NOPLAINTEXT allows non-TLS connections. We check TLS encryption in oauth_server_mech_step() based on configuration.
+	SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS, /* security_flags */
 	SASL_FEAT_WANT_CLIENT_FIRST | SASL_FEAT_ALLOWS_PROXY, /* features */
 	&server_glob_context,  /* glob_context */
 	&oauth_server_mech_new,  /* mech_new */
@@ -596,6 +603,7 @@ int sasl_server_plug_init(
 	oauth_glob_context_t *gctx;
 	int r;
 	const char *val;
+	const char *no_tls;
 	const char *grace;
 	char propname[1024];
 	int propnum = 0;
@@ -613,6 +621,9 @@ int sasl_server_plug_init(
 	gctx = (oauth_glob_context_t *)oauth_server_plugin.glob_context;
 	memset(gctx, 0, sizeof(*gctx));
 
+#ifdef HACK
+	y_init_logs("crudeoauth", Y_LOG_MODE_CONSOLE, Y_LOG_LEVEL_DEBUG, NULL, "Starting SASL OAUTHBEARER");
+#endif
 	if (r_global_init() != RHN_OK) {
 		utils->log(NULL, SASL_LOG_ERR, "OAUTHBEARER r_global_init failed");
 		return SASL_FAIL;
@@ -641,6 +652,17 @@ int sasl_server_plug_init(
 	else
 		gctx->grace = atoi(grace);
 
+
+	/*
+	 * Ignore SSF / TLS check
+	 */
+	r = utils->getopt(utils->getopt_context, "OAUTHBEARER", "oauthbearer_no_tls", &no_tls, NULL);
+	if ((r != 0) || (no_tls == NULL) || (*no_tls == '\0') || atoi(no_tls) != 1) {
+		gctx->tls_required = true;
+	} else {
+		gctx->tls_required = false;
+	}
+
 	/*
 	 * Load the trusted audiences
 	 */
@@ -767,21 +789,41 @@ int sasl_server_plug_init(
 			continue;
 		}
 
-		if ((gctx->trusted_jwks_str = utils->malloc(JWKS_BUFFSIZE)) == NULL) {
-			utils->log(NULL, SASL_LOG_ERR, "cannot allocate memory");
-			return SASL_NOMEM;
-		}
-
-		jwks_fp = fopen(jwks_filename, "r");
-		if (jwks_fp == NULL) {
+		FILE *jwks_fp = fopen(jwks_filename, "r");
+		if (!jwks_fp) {
 			utils->log(NULL, SASL_LOG_ERR, "Failed to open JWKS file \"%s\" (%s)", jwks_filename, strerror(errno));
 			return SASL_CONFIGERR;
 		}
-		if (NULL == fgets(gctx->trusted_jwks_str, JWKS_BUFFSIZE, jwks_fp)) {
+
+		if (fseek(jwks_fp, 0, SEEK_END) != 0) {
+			utils->log(NULL, SASL_LOG_ERR, "Failed to seek JWKS file \"%s\" (%s)", jwks_filename, strerror(errno));
 			fclose(jwks_fp);
-			utils->log(NULL, SASL_LOG_ERR, "Failed to load JWKS from \"%s\"", jwks_filename);
 			return SASL_CONFIGERR;
 		}
+
+		long fsize = ftell(jwks_fp);
+		if (fsize < 0) {
+			utils->log(NULL, SASL_LOG_ERR, "Failed to determine JWKS size \"%s\" (%s)", jwks_filename, strerror(errno));
+			fclose(jwks_fp);
+			return SASL_CONFIGERR;
+		}
+
+		rewind(jwks_fp);
+
+		gctx->trusted_jwks_str = utils->malloc(fsize + 1);
+		if (!gctx->trusted_jwks_str) {
+			utils->log(NULL, SASL_LOG_ERR, "Out of memory reading JWKS \"%s\" (%s)", jwks_filename, strerror(errno));
+			fclose(jwks_fp);
+			return SASL_NOMEM;
+		}
+
+		if (fread(gctx->trusted_jwks_str, 1, fsize, jwks_fp) != (size_t)fsize) {
+			utils->log(NULL, SASL_LOG_ERR, "Failed to read full JWKS \"%s\" (%s)", jwks_filename, strerror(errno));
+			fclose(jwks_fp);
+			return SASL_CONFIGERR;
+		}
+
+		gctx->trusted_jwks_str[fsize] = '\0';
 		fclose(jwks_fp);
 
 		if (!gctx->trusted_jwks_str) {
@@ -1020,7 +1062,8 @@ static void oauth_client_mech_dispose(
 static sasl_client_plug_t oauth_client_plugin = {
 	"OAUTHBEARER", /* mech_name */
 	0, /* max_ssf */
-	SASL_SEC_NOANONYMOUS, /* security_flags */
+	// setting SASL_SEC_NOPLAINTEXT allows non-TLS connections. We check TLS encryption in oauth_server_mech_step() based on configuration.
+	SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS, /* security_flags */
 	SASL_FEAT_WANT_CLIENT_FIRST | SASL_FEAT_ALLOWS_PROXY, /* features */
 	NULL, /* required_prompts */
 	NULL, /* glob_context */
